package algorithms;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;

import backend.Move;
import backend.RobotAlgorithmIF;
import backend.SensorGroup;

/**
 * Algorithms and Compubility -- Project 2 (aka Kung Foo Robots) This algorithm
 * is not really an algorithm at all. Instead, it creates a way to manually
 * control the robot through keyboard input.
 * 
 * The keyboard commands are: 5: ahead with crumbs 8: ahead without crumbs 4:
 * turn left 6: turn right Anything else: skip a turn.
 * 
 * As this uses the given input to get its movements (default is System.in).
 * Each line of the input counts as a new move (if using the console, you have
 * to hit "enter" or "return" after each move). Any characters after the first
 * on a line are ignored.
 * 
 * @author Seth Sweep
 */
public class ManualControl implements RobotAlgorithmIF {

    private BufferedReader _in;

    private BufferedWriter _out;

    /**
     * Creates a new ManualControl with the given input and output sources.
     * 
     * @param in The input source.
     * @param out The output source.
     */
    public ManualControl(Reader in, Writer out) {
        _in = new BufferedReader(in);
        _out = new BufferedWriter(out);
    }

    /**
     * Creates a new ManualControl, using System.in and System.out as the input
     * and output sources. This method is equivalent to calling
     * <code>new ManualControl(new InputStreamReader(System.in), new OutputStreamWriter(
     * System.out)</code>.
     */
    public ManualControl() {
        this(new InputStreamReader(System.in), new OutputStreamWriter(
                System.out));
    }

    /**
     * Gets the next move. Waits for the user to enter in a value, and moves
     * based on that (see commands above).
     * 
     * @throws RuntimeException if the supplied reader or writer throws an
     *         IOException.
     */
    public Move getMove(SensorGroup sensors) {
        printMoveMessage(sensors);
        char input = getKeyboardInput();
        return getMappedMove(input);
    }

    private void printMoveMessage(SensorGroup sensors) {
        try {
            _out.append("__________SENSORS__________\n");
            _out.append(sensors.toString() + "\n");
            _out.append("AWAITING MOVE...\n");
            _out.flush();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private char getKeyboardInput() {
        char input = '1';
        try {
            String inputString = _in.readLine();
            if (inputString.length() != 0) {
                input = inputString.charAt(0);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return input;
    }

    private Move getMappedMove(char input) {
        Move move = Move.SKIP;
        if (input == '5') {
            move = Move.AHEAD_WITH_CRUMBS;
        } else if (input == '8') {
            move = Move.AHEAD_WITHOUT_CRUMBS;
        } else if (input == '4') {
            move = Move.TURN_LEFT;
        } else if (input == '6') {
            move = Move.TURN_RIGHT;
        }
        return move;
    }

    public String getName() {
        return "Manual Control";
    }

    public String toString() {
        return getName();
    }
}
