package backend;

/**
 * This class is an enum representing the valid moves a robot can make in a 
 * game. The options are:
 * 
 * SKIP: skip a turn
 * AHEAD_WITH_CRUMBS: attempt to move ahead while laying breadcrumbs
 * AHEAD_WITHOUT_CRUMBS: attempt to move ahead without laying breadcrumbs
 * TURN_LEFT: turn left
 * TURN_RIGHT: turn right
 */
public enum Move {
    SKIP("skips a turn"), 
    AHEAD_WITH_CRUMBS("attempts to lay breadcrumbs and move ahead"), 
    AHEAD_WITHOUT_CRUMBS("attempts to move ahead"), 
    TURN_LEFT("turns left"), 
    TURN_RIGHT("turns right");

    private String actionString;

    private Move(String actionString) {
        this.actionString = actionString;
    }

    /**
     * Returns a String that represents the action being attempted. Examples are
     * "skips a turn" for SKIP, and "attempts to lay breadcrumbs and move ahead"
     * for AHEAD_WITH_CRUMBS.
     * 
     * @return The action string for the enum.
     */
    public final String getActionString() {
        return actionString;
    }
}