package backend;

/**
 * Algorithms and Compubility -- Project 2 (aka Kung Foo Robots)
 * 
 * This abstract class defines the interface for all robot moving algorithms.
 * 
 * This class has been deprecated, but is maintained for backward compatability
 * with previous years' algorithms. New algorithms should instead directly
 * implement RobotAlgorithmIF.
 * 
 * @author Seth Sweep
 * @deprecated Use RobotAlgorithmIF instead.
 */
public abstract class RobotAlgorithm implements RobotAlgorithmIF {

    /**
     * Constant representing "skip a turn" move.
     */
    public final static int SKIP = 0;

    /**
     * Constant representing "move ahead with crumbs" move.
     */
    public final static int AHEAD_WITH_CRUMBS = 1;

    /**
     * Constant representing "move ahead without crumbs" move.
     */
    public final static int AHEAD_WITHOUT_CRUMBS = 2;

    /**
     * Constant representing "turn left" move.
     */
    public final static int TURN_LEFT = 3;

    /**
     * Constant representing "turn right" move.
     */
    public final static int TURN_RIGHT = 4;

    
    /**
     * Shorthand for SKIP.
     */
    public final static int S = SKIP;

    /**
     * Shorthand for AHEAD_WITH_CRUMBS.
     */
    public final static int AWC = AHEAD_WITH_CRUMBS;

    /**
     * Shorthand for AHEAD_WITHOUT_CRUMBS.
     */
    public final static int A = AHEAD_WITHOUT_CRUMBS;

    /**
     * Shorthand for TURN_LEFT.
     */
    public final static int TL = TURN_LEFT;

    /**
     * Shorthand for TURN_RIGHT.
     */
    public final static int TR = TURN_RIGHT;


    /**
     * Maps the previous int-based moves to their new enum equivalents, where
     * the (0-based) array position is the same as the previous value.
     */
    private static final Move[] moveMap = new Move[] { Move.SKIP,
            Move.AHEAD_WITH_CRUMBS, Move.AHEAD_WITHOUT_CRUMBS, Move.TURN_LEFT,
            Move.TURN_RIGHT };

    /**
     * The sensor data for the current turn is stored in this variable by the
     * game prior to each call to getMove().  The seven sensors (in the order
     * they appear in the array) are:
     * 
     * Center, Left, Ahead-left, Ahead-one, Ahead-right, Right, and Ahead (the
     * long-distance sensor).
     * 
     * Each String consists of the sensor name, followed by the sensor input in
     * the following format:
     * 
     * SensorName: SensorData
     * 
     * An example of sensor input is:
     * 
     * Ahead-right: I detect an unclaimed building worth 1 point(s) 
     * 
     * If the sensor is the far sensor, the input will also display the
     * distance, as in this example:
     * 
     * Ahead: I detect a wall 50 spaces away 
     */
    protected String[] _sensorData = new String[6];

    /**
     * This method sets the sensor data to the given String array.  This
     * implementation merely stores sensorData into the _sensorData protected
     * field.
     * 
     * @param sensorData The sensor data to set.
     */
    public void setSensors(String[] sensorData) {
        _sensorData = sensorData;
    }

    /**
     * The new getMove() call, as defined in RobotAlgorithmIF. This method
     * translates the new call into the old calls.
     */
    public Move getMove(SensorGroup sensorData) {
        setSensors(sensorData.toStringArray());
        int moveInt = getMove();
        return moveMap[moveInt];
    }

    /**
     * Returns toString() as a name, which was the previous method of
     * determining algorithm names).
     */
    public String getName() {
        return toString();
    }

    /**
     * Returns the move the robot should make. Prior to this being called,
     * _sensorData will be set to the current sensor input via a call to
     * setSensors.
     * 
     * @return The chosen move for this turn. Valid moves values are any of the
     *         static int fields. Any other value will be interpreted as "SKIP".
     */
    public abstract int getMove();
}
