package backend;

/**
 * Algorithms and Computability Robot Game Project (aka Kung Foo Robots)
 * 
 * This is the common interface for all robot moving algorithms.
 * 
 * @author Matthew Justin, Matthew Carlson, and Jon Quarfoth
 */
public interface RobotAlgorithmIF {
    /**
     * The simple method you must implement.
     * 
     * This method is called every time your robot takes a turn. Return the move
     * (selected from the Move enum at backend.Move) you want your robot to
     * make. Possible moves are:
     * 
     * Move.SKIP (skips a turn)
     * Move.AHEAD_WITH_CRUMBS (lay breadcrumbs and attempt to move ahead)
     * Move.AHEAD_WITHOUT_CRUMBS (attempts to move ahead without laying breadcrumbs)
     * Move.TURN_LEFT (turn left)
     * Move.TURN_RIGHT (turns right)
     * 
     * @param sensors The sensor data received by your robot this turn.
     * @return The chosen move for this turn. (See the Move enum.)
     */
    public Move getMove(SensorGroup sensors);

    /**
     * Returns the name of your robot. The game uses this in displaying
     * information about your robot.
     * 
     * @return the name of your robot.
     */
    public String getName();
}
