package backend;

/**
 * Class that generates SensorGroups for a given playing field and robot. Newly
 * created SensorGroups will reflect the state to the underlying PlayingField at
 * their moment of creation.
 * 
 * @author Matthew Justin
 */
public class SensorGenerator {
    private PlayingField _playingField;

    private Robot _robot;

    /**
     * Create a new generator for a given playing field and robot.
     * 
     * @param playingField The field to use.
     * @param robot The robot to use.
     */
    public SensorGenerator(PlayingField playingField, Robot robot) {
        _playingField = playingField;
        _robot = robot;
    }

    /**
     * Return a new SensorGroup based on the robot's current position.
     * 
     * @return A SensorGroup.
     */
    public SensorGroup getSensors() {
        return new SensorGroup(_playingField, _robot);
    }
}
