package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JPanel;

import backend.GameLogic;

/**
 * JFrame which displays a PlayingFieldPanel and an InfoPanel to show the status
 * of a GameLogic as it progresses.
 * 
 * @author Seth Sweep
 */
public class RoboGameFrame extends JFrame {

    private GameLogic _gameLogic;

    /**
     * Creates a new RoboGameFrame for the given GameLogic. Upon construction
     * this will add itself as a listener to the game.
     * 
     * @param gameLogic The GameLogic serving as the model.
     */
    public RoboGameFrame(GameLogic gameLogic) {
        _gameLogic = gameLogic;
        setUpGamePanel();
        setWindowSettings();
    }

    private void setUpGamePanel() {
        PlayingFieldPanel fieldPanel = new PlayingFieldPanel(_gameLogic);
        fieldPanel.setPreferredSize(new Dimension(defaultWidth(),
                defaultHeight()));

        InfoPanel infoPanel = new InfoPanel(_gameLogic);
        infoPanel.setPreferredSize(new Dimension(defaultWidth(),
                InfoPanel.PANEL_HEIGHT));

        JPanel gamePanel = new JPanel(new BorderLayout());
        gamePanel.add(fieldPanel, BorderLayout.CENTER);
        gamePanel.add(infoPanel, BorderLayout.SOUTH);

        getContentPane().add(gamePanel);
    }

    private int defaultHeight() {
        return 10 * _gameLogic.getPlayingField().getHeight();
    }

    private int defaultWidth() {
        return 10 * _gameLogic.getPlayingField().getWidth();
    }

    private void setWindowSettings() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setResizable(true);
        setFocusableWindowState(true);
        setTitle("Kung Foo Robots! -- Level " + _gameLogic.getLevel());
        pack();
        setLocationRelativeTo(null);
    }

}
