package backend;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import testutils.TestUtils;

/**
 * @author sweepst
 */
public class BuildingCellTest extends TestCase {

    /**
     * Constructor for OwnableCellTest.
     * 
     * @param arg0
     */
    public BuildingCellTest(String arg0) {
        super(arg0);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    public void testSetAndGetPoints() {
        BuildingCell buildingCell = new BuildingCell();
        assertEquals(
                "The point value of the building cell should be the default point value.",
                BuildingCell.DEFAULT_POINTS, buildingCell.getPoints());
        buildingCell.setPoints(5);
        assertEquals("The point value of the building cell should be 5.", 5,
                buildingCell.getPoints());
        buildingCell.setPoints(3);
        assertEquals("The point value of the building cell should be 3.", 3,
                buildingCell.getPoints());
    }

    /**
     * Tests getOwner() and setOwner() for legal owner values.
     */
    public void testSetAndGetOwner() {
        BuildingCell buildingCell = new BuildingCell();
        assertEquals("The owner of the building cell should be nobody.",
                null, buildingCell.getOwner());
        testLegalGetAndSetOwner(buildingCell, null);
        testLegalGetAndSetOwner(buildingCell, new Robot(0));
        testLegalGetAndSetOwner(buildingCell, new Robot(1));
        testLegalGetAndSetOwner(buildingCell, new Robot(100));
    }

    /**
     * Tests the equals() and hashCode() methods.
     */
    public void testEqualsAndHashCode() throws Exception {
        BuildingCell b = new BuildingCell();
        StreetCell s = new StreetCell();
        assertFalse(b.equals(s));

        Robot[] robots = new Robot[]{null, new Robot(0), new Robot(1), new Robot(1)};
        List<BuildingCell> cells1 = createCellList(robots);
        List<BuildingCell> cells2 = createCellList(robots);
        TestUtils.assertEqualsAndHashCodes(cells1, cells2);
    }
	
	public void testToString() {
		List<BuildingCell> list = createCellList();
		for (BuildingCell c : list) {
			String expected = String.format(
					"Building cell: points: %d; owner: %s", c.getPoints(), c
							.getOwner());
			assertEquals("The wrong String was returned", expected, c.toString());
		}
	}
    
    public void testIsOwned() {
        BuildingCell buildingCell = new BuildingCell();
        assertFalse("The building cell should be unowned", buildingCell.isOwned());
        
        buildingCell.setOwner(new Robot(1));
        assertTrue("The building cell should be owned", buildingCell.isOwned());
        
        buildingCell.setOwner(new Robot(2));
        assertTrue("The building cell should be owned", buildingCell.isOwned());
        
        buildingCell.setOwner(null);
        assertFalse("The building cell should be unowned", buildingCell.isOwned());
        
        buildingCell.setOwner(new Robot(1));
        assertTrue("The building cell should be owned", buildingCell.isOwned());
    }

    private List<BuildingCell> createCellList() {
        Robot[] robots = new Robot[3];
        robots[0] = null;
        for (int i = 1; i < robots.length; i++) {
            robots[i] = new Robot(i - 1);
        }
        return createCellList(robots);
    }

    private List<BuildingCell> createCellList(Robot[] robots) {
		List<BuildingCell> result = new ArrayList<BuildingCell>();
        for (Robot owner: robots) {
            for (int points = 1; points < 3; points++) {
                BuildingCell cell = newBuildingCell(owner, points);
                result.add(cell);
            }
        }
		return result;
    }

    private void testLegalGetAndSetOwner(BuildingCell buildingCell, Robot owner) {
        buildingCell.setOwner(owner);
        assertEquals(String.format(
                "The owner of the building cell should be %s.", owner), owner,
                buildingCell.getOwner());
    }

    private BuildingCell newBuildingCell(Robot owner, int points) {
        BuildingCell cell = new BuildingCell(points);
        cell.setOwner(owner);
        return cell;
    }

    //TODO Write test for isOwned.
}
