package backend;

import java.util.Arrays;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Random;

import junit.framework.TestCase;
import testutils.TestUtils;

/**
 * @author justnm
 */
public class DirectionTest extends TestCase {

    private int numTests = 1000;

    public void testDirections() {
        final String message = "The wrong direction was returned";
        Direction[] directions = new Direction[] { Direction.NORTH,
                Direction.EAST, Direction.SOUTH, Direction.WEST };
        assertEquals("The wrong directions were returned",
                new HashSet<Direction>(Arrays.asList(directions)),
                EnumSet.allOf(Direction.class));

        Direction current = Direction.WEST;
        for (int i = 0; i < 4; i++) {
            current = current.right();
            assertEquals(message, directions[i], current);
        }

        current = Direction.NORTH;
        for (int i = 3; i >= 0; i--) {
            current = current.left();
            assertEquals(message, directions[i], current);
        }

        for (int i = 0; i < 4; i++) {
            assertEquals(message, directions[(i + 2) % 4], directions[i].back());
        }

    }

    public void testRandomDirection() {
        Direction[] results = new Direction[numTests];
        Random random = new Random();
        for (int i = 0; i < results.length; i++) {
            results[i] = Direction.randomDirection(random);
        }
        TestUtils.verifyRandomness(results, Direction.values());
    }
    
    public void testToString() {
        final String message = "The wrong String value was returned";
        assertEquals(message, "north", Direction.NORTH.toString());
        assertEquals(message, "east", Direction.EAST.toString());
        assertEquals(message, "south", Direction.SOUTH.toString());
        assertEquals(message, "west", Direction.WEST.toString());
    }

}
