package backend;

import java.util.Arrays;
import java.util.Random;

import junit.framework.TestCase;
import robots.PredictableTestRobot;

public class FullGameTests extends TestCase {
    private static final long RANDOM_SEED = 5465465645L;

    public void testLevel1SinglePlayer() {
        int level = 1;
        gameTest(1, level, new int[] { 352 }, 5000);
    }

    public void testLevel1Multiplayer() {
        Random random = new Random(RANDOM_SEED);
        gameTest(2, 1, new int[] { 313, 312 }, 4439);
        random.setSeed(RANDOM_SEED);
        gameTest(4, 1, new int[] { 157, 156, 156, 156 }, 4439);
        random.setSeed(RANDOM_SEED);
        gameTest(6, 1, new int[] { 93, 93, 132, 132, 86, 89 }, random, 4439);
    }

    public void testLevel2SinglePlayer() {
        Random random = new Random(RANDOM_SEED);
        gameTest(1, 2, new int[] { 314 }, random, 5000);
    }

    public void testLevel2Multiplayer() {
        Random random = new Random(RANDOM_SEED);
        gameTest(2, 2, new int[] { 314, 408 }, random, 5000);
        random.setSeed(RANDOM_SEED);
        gameTest(4, 2, new int[] { 258, 273, 361, 251 }, random, 5000);
        random.setSeed(RANDOM_SEED);
        gameTest(6, 2, new int[] { 272, 145, 267, 268, 50, 1 }, random, 5000);
    }

    public void testLevel3SinglePlayer() {
        Random random = new Random(RANDOM_SEED);
        gameTest(1, 3, new int[] { 40 }, random, 5000);
    }

    public void testLevel3Multiplayer() {
        Random random = new Random(RANDOM_SEED);
        gameTest(2, 3, new int[] { 40, 40 }, random, 5000);
        random.setSeed(RANDOM_SEED);
        gameTest(4, 3, new int[] { 40, 40, 4, 35 }, random, 5000);
        random.setSeed(RANDOM_SEED);
        gameTest(6, 3, new int[] { 40, 40, 4, 35, 0, 0 }, random, 5000);
    }

    private void gameTest(int numRobots, int level, int[] expected,
            Random random, int turns) {
        RobotAlgorithmIF[] robots = new RobotAlgorithmIF[numRobots];
        for (int i = 0; i < numRobots; i++) {
            robots[i] = new PredictableTestRobot();
        }
        GameLogic gl = new GameLogic(robots, level, random);
        assertArraysEquals("The wrong scores were returned", expected, gl
                .execute());
        assertEquals("The game ended on the wrong turn", turns, gl
                .getCurrentTurn());
    }

    private void gameTest(int numRobots, int level, int[] expected, int turns) {
        gameTest(numRobots, level, expected, null, turns);
    }

    private static void assertArraysEquals(String message, int[] expected,
            int[] actual) {
        String expectedString = message
                + String.format(" expected:<%s> but was:<%s>", Arrays
                        .toString(expected), Arrays.toString(actual));
        assertTrue(expectedString, Arrays.equals(expected, actual));
    }

}
