package backend;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

public class RecursiveEnclosureScannerTest extends TestCase {
    //TODO write test for 1x1 capture starting to the side of the block.
    
	private PlayingField _field;
    
    private Robot _robot0 = new Robot(0);
    private Robot _robot1 = new Robot(1);
    
    private List<Robot> _robots = new ArrayList<Robot>();

	@Override
	protected void setUp() throws Exception {
		super.setUp();
        _robots = Arrays.asList(_robot0, _robot1);
		_field = new PlayingField(_robots, 1, null);
	}

	public void testCellsBecomeUnchecked() {
        Robot robot = new Robot(0);
		RecursiveEnclosureScanner scanner = new RecursiveEnclosureScanner(
				_field, robot);
		scanner.awardPoints();
		PlayingFieldTest.assertAllCellsUnchecked(_field);
	}

	public void testRegularEnclosure() {
        RecursiveEnclosureScanner scanner0 = new RecursiveEnclosureScanner(
                _field, _robot0);
        RecursiveEnclosureScanner scanner1 = new RecursiveEnclosureScanner(
                _field, _robot1);

        createSimpleEnclosure(2, 4, 10, 10, _robot0);

        assertEnclosureScanner(scanner0, 25);
        assertEnclosureScanner(scanner0, 0);
        assertEnclosureScanner(scanner1, 0);
    }

	public void testTwoEnclosures() {
		RecursiveEnclosureScanner scanner0 = new RecursiveEnclosureScanner(
				_field, _robot0);
        RecursiveEnclosureScanner scanner1 = new RecursiveEnclosureScanner(
                _field, _robot1);
        
		createSimpleEnclosure(4, 4, 8, 8, _robot0);
		createSimpleEnclosure(24, 24, 4, 4, _robot1);

		assertEnclosureScanner(scanner0, 16);
		assertEnclosureScanner(scanner0, 0);
		assertEnclosureScanner(scanner1, 4);
		assertEnclosureScanner(scanner0, 0);
		assertEnclosureScanner(scanner1, 0);
	}

	public void testIrregularEnclosure() {
		for (int y = 0; y <= 10; y++) {
			setTrailOfCell(0, y, _robot0);
		}
		for (int y = 0; y <= 6; y++) {
			setTrailOfCell(10, y, _robot0);
		}
		for (int y = 6; y <= 10; y++) {
			setTrailOfCell(6, y, _robot0);
		}
		for (int x = 0; x <= 10; x++) {
			setTrailOfCell(x, 0, _robot0);
		}
		for (int x = 0; x <= 6; x++) {
			setTrailOfCell(x, 10, _robot0);
		}
		for (int x = 6; x <= 10; x++) {
			setTrailOfCell(x, 6, _robot0);
		}
		RecursiveEnclosureScanner scanner = new RecursiveEnclosureScanner(
				_field, _robot0);
		assertEnclosureScanner(scanner, 21);
	}

	private void setTrailOfCell(int x, int y, Robot robot) {
		((StreetCell) _field.getCell(x, y)).setRobotTrail(robot, true);
	}

	private void createSimpleEnclosure(int xStart, int yStart, int width,
			int height, Robot robot) {
		_field.setRobot(robot, Direction.NORTH, xStart, yStart);

		for (int y = yStart; y <= yStart + height; y++) {
			setTrailOfCell(xStart, y, robot);
			setTrailOfCell(xStart + width, y, robot);
		}
		for (int x = xStart; x <= xStart + width; x++) {
			setTrailOfCell(x, yStart, robot);
			setTrailOfCell(x, yStart + height, robot);
		}
	}

    private void assertEnclosureScanner(RecursiveEnclosureScanner scanner,
            int expected) {
        assertEquals("The wrong point value was returned", expected, scanner
                .awardPoints());
        PlayingFieldTest.assertAllCellsUnchecked(_field);
    }
}
