package backend;

import junit.framework.TestCase;
import robots.MockRobotAlgorithm;
import testutils.TestUtils;

public class RobotAlgorithmTest extends TestCase {
    MockRobotAlgorithm testAlgorithm = new MockRobotAlgorithm();

    public void testGetName() {
        testGetName("test");
        testGetName("foo");
    }

    public void testConstantTranslation() {
        testConstantTranslation(RobotAlgorithm.AHEAD_WITH_CRUMBS,
                Move.AHEAD_WITH_CRUMBS);
        testConstantTranslation(RobotAlgorithm.AHEAD_WITHOUT_CRUMBS,
                Move.AHEAD_WITHOUT_CRUMBS);
        testConstantTranslation(RobotAlgorithm.SKIP, Move.SKIP);
        testConstantTranslation(RobotAlgorithm.TURN_LEFT, Move.TURN_LEFT);
        testConstantTranslation(RobotAlgorithm.TURN_RIGHT, Move.TURN_RIGHT);
    }

    public void testGetMove() {
        PlayingField field = new PlayingField(1, 1);
        SensorGroup sensors = new SensorGroup(field, field.getRobots().get(0));
        testAlgorithm.getMove(sensors);
        TestUtils.assertArraysEqual("The sensors should have been updated",
                sensors.toStringArray(), testAlgorithm.getSensorData());
    }

    public void testSetSensors() {
        String[] sensors = new String[] { "foo", "bar", "baz", "four", "five",
                "six", "longrange" };
        testAlgorithm.setSensors(sensors);
        TestUtils.assertArraysEqual("The sensors should have been updated",
                sensors, testAlgorithm.getSensorData());
    }

    public void testIntConstants() {
        final String message = "The constants weren't their expected values";
        Integer[] intVals = { 1, 2, 0, 3, 4 };
        Integer[] longConstants = { RobotAlgorithm.AHEAD_WITH_CRUMBS,
                RobotAlgorithm.AHEAD_WITHOUT_CRUMBS, RobotAlgorithm.SKIP,
                RobotAlgorithm.TURN_LEFT, RobotAlgorithm.TURN_RIGHT };
        Integer[] shortConstants = { RobotAlgorithm.AWC, RobotAlgorithm.A,
                RobotAlgorithm.S, RobotAlgorithm.TL, RobotAlgorithm.TR };
        TestUtils.assertArraysEqual(message, intVals, longConstants);
        TestUtils.assertArraysEqual(message, shortConstants, longConstants);
    }

    private void testGetName(String name) {
        testAlgorithm.setToString(name);
        assertEquals("The Algorithm's name should equal its toString method",
                testAlgorithm.getName(), testAlgorithm.toString());
        assertEquals("The Algorithm's name should equal its toString method",
                testAlgorithm.getName(), name);
    }

    private void testConstantTranslation(int intConstant, Move enumConstant) {
        testAlgorithm.setNextMove(intConstant);
        PlayingField field = new PlayingField(1, 1);
        SensorGroup sensors = new SensorGroup(field, field.getRobots().get(0));
        assertEquals(
                "The int constant wasn't translated into the right enum constant",
                enumConstant, testAlgorithm.getMove(sensors));
    }
}
