package backend;

import java.awt.Point;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;
import testutils.TestUtils;

public class SensorTest extends TestCase {
    
    /*
     * sensorData[0] = "Center: " + getSensorSquare(x, y, player); sensorData[1] =
     * "Left: " + getSensorSquare(x - 1, y, player); sensorData[2] =
     * "Ahead-left: " + getSensorSquare(x - 1, y - 1, player); sensorData[3] =
     * "Ahead-one: " + getSensorSquare(x, y - 1, player); sensorData[4] =
     * "Ahead-right: " + getSensorSquare(x +1, y - 1, player); sensorData[5] =
     * "Right: " + getSensorSquare(x + 1, y, player); sensorData[6] = "Ahead: " +
     * getSensorSquare(x, getSensorCollision(player, loc, x, y), player) + (y -
     * getSensorCollision(player, loc, x, y) - 1) + " spaces away ";
     */
    public void testGetSensorData() {
        final String crumblessRoad = "I detect a road with no breadcrumbs ";
        final String myCrumbsRoad = "I detect a road with my breadcrumbs ";
        final String adversaryCrumbsRoad = "I detect a road with my adversary's breadcrumbs ";
        final String bothCrumbsRoad = "I detect a road with my breadcrumbs and my adversary's breadcrumbs ";
        final String wall = "I detect a wall ";
        final String adversary = "and my adversary ";

        Robot robot0 = new Robot(0);
        Robot robot1 = new Robot(1);
        List<Robot> robots = Arrays.asList(robot0, robot1);
        PlayingField field = new PlayingField(robots, 1, null);
        
        SensorGenerator sensor0 = new SensorGenerator(field, robot0);
        SensorGenerator sensor1 = new SensorGenerator(field, robot1);

        String[] p0expected = { crumblessRoad, wall, wall, crumblessRoad,
                buildingString(1), crumblessRoad, wall };
        String[] p1expected = p0expected;

        testGetSensorData(sensor0, p0expected, 50);
        testGetSensorData(sensor1, p1expected, 50);

        field.setRobot(robot1, Direction.NORTH, 0, 11);
        ((StreetCell) field.getCell(1, 10)).setRobotTrail(robot0, true);
        ((StreetCell) field.getCell(0, 10)).setRobotTrail(robot1, true);
        ((BuildingCell) field.getCell(1, 11)).setPoints(3);

        p1expected = new String[] { crumblessRoad, wall, wall, myCrumbsRoad,
                adversaryCrumbsRoad, buildingString(3),
                crumblessRoad + adversary };

        testGetSensorData(sensor0, p0expected, 50);
        testGetSensorData(sensor1, p1expected, 10);

        ((StreetCell) field.getCell(1, 10)).setRobotTrail(robot1, true);
        ((StreetCell) field.getCell(0, 0)).setRobotTrail(robot1, true);
        ((BuildingCell) field.getCell(1, 11)).setPoints(30);
        ((BuildingCell) field.getCell(1, 11)).setOwner(robot0);
        ((BuildingCell) field.getCell(1, 1)).setOwner(robot0);
        
        p0expected[0] = adversaryCrumbsRoad;
        p0expected[4] = myBuildingString(1);
        p1expected = new String[] { crumblessRoad, wall, wall, myCrumbsRoad,
                bothCrumbsRoad, adversaryBuildingString(30),
                myCrumbsRoad + adversary };
        
        testGetSensorData(sensor0, p0expected, 50);
        testGetSensorData(sensor1, p1expected, 10);
        
        ((BuildingCell) field.getCell(1, 11)).setOwner(robot1);
        ((BuildingCell) field.getCell(1, 1)).setOwner(robot1);

        p0expected[4] = adversaryBuildingString(1);
        p1expected[5] = myBuildingString(30);
        
        testGetSensorData(sensor0, p0expected, 50);
        testGetSensorData(sensor1, p1expected, 10);

        field.setRobotLocation(robot0, new Point(49, 0));
        
        p0expected = new String[] { crumblessRoad, wall, wall, crumblessRoad,
                crumblessRoad, buildingString(1), wall };
        p1expected[6] = wall;
        
        testGetSensorData(sensor0, p0expected, 1);
        testGetSensorData(sensor1, p1expected, 11);
        
        field.setRobotLocation(robot0, new Point(50, 0));
        
        p0expected = new String[] { crumblessRoad, wall, wall, wall,
                wall, crumblessRoad, wall };
        
        testGetSensorData(sensor0, p0expected, 0);
        testGetSensorData(sensor1, p1expected, 11);
    }

    /**
     * Tests that a SensorGenerator updates its position as the field does,
     * but that a SensorGroup does not.
     *
     */
    public void testSensorGenerator() {
        Robot robot0 = new Robot(0);
        Robot robot1 = new Robot(1);
        List<Robot> robots = Arrays.asList(robot0, robot1);
        PlayingField field = new PlayingField(robots, 1, null);
        field.setRobot(robot1, Direction.NORTH, 0, 11);

        SensorGroup initialSensorData = new SensorGroup(field, robot0);
        SensorGenerator generator = new SensorGenerator(field, robot0);
        String initialSensorString = new SensorGroup(field, robot0).toString();

        final String message = "The wrong SensorGroup was returned";
        assertEquals(message, generator.getSensors().toString(),
                initialSensorString);
        assertEquals(message, new SensorGenerator(field, robot1).getSensors()
                .toString(), new SensorGroup(field, robot1).toString());

        field.setRobot(robot0, Direction.SOUTH, 0, 18);
        assertEquals(message, generator.getSensors().toString(),
                new SensorGroup(field, robot0).toString());
        assertEquals(message, initialSensorData.toString(), initialSensorString);
    }

    private void testGetSensorData(SensorGenerator sensor, String[] expected, int farSensorDistance) {
        final String away = "%d spaces away ";
        final String[] dirs = new String[] { "Center", "Left", "Ahead-left",
                "Ahead-one", "Ahead-right", "Right", "Ahead" };

        String[] expectedCopy = (String[]) expected.clone();

        for (int i = 0; i < expectedCopy.length; i++) {
            expectedCopy[i] = dirs[i] + ": " + expectedCopy[i];
        }

        expectedCopy[6] += String.format(away, farSensorDistance);

        final String message = "The wrong sensor data was returned";

        TestUtils.assertArraysEqual(message, expectedCopy, sensor.getSensors()
                .toStringArray());
        
        testIndividualSensors(sensor, farSensorDistance, expected);
        testToString(sensor, expectedCopy);
    }

    private void testToString(SensorGenerator sensor, String[] expectedCopy) {
        String expected = expectedCopy[0] + "\n" + expectedCopy[1] + "\n"
                + expectedCopy[2] + "\n" + expectedCopy[3] + "\n"
                + expectedCopy[4] + "\n" + expectedCopy[5] + "\n"
                + expectedCopy[6];
        assertEquals("The wrong string value was returned", expected, sensor
                .getSensors().toString());
    }

    private SensorGroup testIndividualSensors(SensorGenerator sensor,
            int farSensorDistance, String[] expected) {
        final String message = "The wrong sensor data was returned";
        SensorGroup sensors = sensor.getSensors();
        assertEquals(message, expected[0], sensors.center().toString());
        assertEquals(message, expected[1], sensors.left().toString());
        assertEquals(message, expected[2], sensors.aheadLeft().toString());
        assertEquals(message, expected[3], sensors.aheadOne().toString());
        assertEquals(message, expected[4], sensors.aheadRight().toString());
        assertEquals(message, expected[5], sensors.right().toString());
        assertEquals(message, expected[6], sensors.farAhead().toString());
        assertEquals(message, farSensorDistance, sensors.farSensorDistance());
        return sensors;
    }

    private static String buildingString(final int pointValue) {
        final String building = "I detect an unclaimed building worth %d point(s) ";
        return String.format(building, pointValue);
    }

    private String adversaryBuildingString(final int pointValue) {
        final String building = "I detect a building belonging to my adversary worth %d point(s) ";
        return String.format(building, pointValue);
    }

    private String myBuildingString(final int pointValue) {
        final String building = "I detect a building belonging to me worth %d point(s) ";
        return String.format(building, pointValue);
    }
}
