package backend;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import testutils.TestUtils;
import exceptions.MultipleRobotsInOnePlaceException;

/**
 * Algorithms and Compubility -- Project 2 (aka Kung Foo Robots)
 * 
 * Tests the street cell.
 * 
 * @author Seth Sweep
 */
public class StreetCellTest extends TestCase {

    /**
     * Constructor for StreetCellTest.
     * 
     * @param arg0
     */
    public StreetCellTest(String arg0) {
        super(arg0);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
    }

    /**
     * Tests the constructor.
     */
    public void testConstructor() {
        Robot robot = new Robot(0);
        StreetCell cell = new StreetCell();
        assertFalse(cell.isOccupied());
        assertEquals(cell.getRobotOccupying(), null);
        assertFalse(cell.isCrumby());
        assertFalse(cell.hasRobotTrail(robot));
        assertFalse(cell.isCrumbyExceptFor(robot));
    }

    /**
     * Tests getRobotTrail() and setRobotTrail().
     */
    public void testSetAndGetRobotTrail() {
        final int players = 2;
        StreetCell streetCell = new StreetCell();
        for (int i = 0; i < players; i++) {
            Robot robot = new Robot(i);
            testGetAndSetRobotTrail(streetCell, robot, false);
            testGetAndSetRobotTrail(streetCell, robot, false);
            testGetAndSetRobotTrail(streetCell, robot, true);
            testGetAndSetRobotTrail(streetCell, robot, true);
            testGetAndSetRobotTrail(streetCell, robot, false);
        }
    }

    /**
     * Tests the isCrumby() and isCrumbyExceptForMe() methods.
     */
    public void testIsCrumbyMethods() {
        StreetCell cell = new StreetCell();
        Robot robot0 = new Robot(0);
        Robot robot1 = new Robot(1);
        Robot[] robots = {robot0, robot1};

        cell.setRobotTrail(robot0, false);
        testIsCrumbyMethods(cell, false, new boolean[] { false, false }, robots);

        cell.setRobotTrail(robot0, true);
        testIsCrumbyMethods(cell, true, new boolean[] { false, true }, robots);

        cell.setRobotTrail(robot1, true);
        testIsCrumbyMethods(cell, true, new boolean[] { true, true }, robots);

        cell.setRobotTrail(robot0, false);
        testIsCrumbyMethods(cell, true, new boolean[] { true, false }, robots);

        cell.setRobotTrail(robot1, false);
        testIsCrumbyMethods(cell, false, new boolean[] { false, false }, robots);
    }

    /**
     * Tests the various occupied methods: getOccupied(), setOccupied(),
     * isOccupied(), and getOccupier().
     */
    public void testOccupiedMethods() {
        StreetCell streetCell = new StreetCell();

        List<Robot> robots = new ArrayList<Robot>();
        robots.add(new Robot(0));
        robots.add(new Robot(0));
        for (Robot robot : robots) {
            testGetAndSetOccupied(streetCell, robot, false, robots);
            testGetAndSetOccupied(streetCell, robot, false, robots);
            testGetAndSetOccupied(streetCell, robot, true, robots);
            testGetAndSetOccupied(streetCell, robot, true, robots);
            testGetAndSetOccupied(streetCell, robot, false, robots);
        }

        streetCell.setRobotOccupying(new Robot(0), true);
        try {
            streetCell.setRobotOccupying(new Robot(0), true);
            fail("A MultipleRobotsInOnePlaceException should have been thrown");
        } catch (MultipleRobotsInOnePlaceException e) {
            // this should happen
        }
    }

    /**
     * Tests the ability to hold multiple robot trails and multiple robots held
     * in one square.
     */
    public void testScalableRobots() {
        // test multiple robot trails
        for (int i = 1; i < 25; i++) {
            StreetCell streetCell = new StreetCell();
            for (int j = 0; j < i; j++) {
                Robot robot = new Robot(j);
                assertFalse("Robot " + j + "'s trail should be false here.",
                        streetCell.hasRobotTrail(robot));
                streetCell.setRobotTrail(robot, true);
                assertTrue("Robot " + j + "'s trail should be true here.",
                        streetCell.hasRobotTrail(robot));
            }
        }
    }

    /**
     * Tests the equals() and hashCode() methods.
     */
    public void testEqualsAndHashCode() throws Exception {
        BuildingCell b = new BuildingCell();
        StreetCell s = new StreetCell();
        assertFalse(s.equals(b));

        List<StreetCell> cells1 = new ArrayList<StreetCell>();
        List<StreetCell> cells2 = new ArrayList<StreetCell>();
        createCellLists(cells1, cells2);
        TestUtils.assertEquals(cells1, cells2);
    }
    
    public void testToString() {
        Robot robot = new Robot(2);
        StreetCell cell = new StreetCell();
        cell.setChecked(false);
        cell.setRobotOccupying(robot, true);
        cell.setRobotTrail(new Robot(1), true);
        assertEquals("The wrong String was returned",
                "StreetCell: trails: [1] occupier: 2 checked: false", cell
                        .toString());
        
        cell.setChecked(true);
        cell.setRobotOccupying(robot, false);
        cell.setRobotTrail(new Robot(8), true);
        assertEquals("StreetCell: trails: [1, 8] occupier: none checked: true", cell
                        .toString());
    }
    
    public void testGetObstacleMessage() {
        StreetCell cell = new StreetCell();
        assertEquals(
                "The wrong message was returned",
                "Game Logic: Robot blocked by another robot.  \"You shall not pass!\" it exclaims.",
                cell.getObstacleMessage());
    }

    private void testGetAndSetRobotTrail(StreetCell cell, Robot robot,
            boolean boolVal) {
        final String message = String.format(
                "Robot %s's trail should have been %s here.", robot, boolVal);
        cell.setRobotTrail(robot, boolVal);
        assertTrue(message, boolVal == cell.hasRobotTrail(robot));
    }

    private void testGetAndSetOccupied(StreetCell cell, Robot robot,
            boolean boolVal, List<Robot> players) {
        final String message = String.format(
                "Robot %s's occupying should be %s here.", robot, boolVal);
        cell.setRobotOccupying(robot, boolVal);
        assertEquals(message, boolVal, cell.getRobotOccupying() == robot);

        assertOtherRobotsNotOccupying(cell, robot, players);

        assertEquals("The square had the wrong occupied value", cell
                .isOccupied(), boolVal);

        Robot expectedOccupier = (boolVal ? robot : null);
        assertEquals("The square returned the wrong occupier", expectedOccupier,
                cell.getRobotOccupying());

    }

    private void assertOtherRobotsNotOccupying(StreetCell cell, Robot robot,
            List<Robot> players) {
        for (Robot otherRobot : players) {
            if (otherRobot != robot) {
                assertFalse(
                        "The other robots shouldn't be occupying this square: "+
                        robot + ", " + otherRobot,
                        cell.getRobotOccupying() == otherRobot);
            }
        }
    }

    private void testIsCrumbyMethods(StreetCell cell, boolean isCrumby,
            boolean[] isCrumbyExceptFor, Robot[] robots) {
        final String message = "The wrong isCrumby value was returned";
        assertEquals(message, isCrumby, cell.isCrumby());
        for (int i = 0; i < isCrumbyExceptFor.length; i++) {
            assertEquals(message, cell.isCrumbyExceptFor(robots[i]),
                    isCrumbyExceptFor[i]);
        }
    }

    private StreetCell newStreetCell(Robot[] robots, boolean crumb0, boolean crumb1,
            Robot occupier) {
        StreetCell result = new StreetCell();
        if (occupier != null) {
            result.setRobotOccupying(occupier, true);
        }
        result.setRobotTrail(robots[0], crumb0);
        result.setRobotTrail(robots[1], crumb1);
        return result;
    }

    private boolean booleanVal(int val) {
        return (val == 0 ? false : true);
    }

    private void createCellLists(List<StreetCell> cells1,
            List<StreetCell> cells2) {
        Robot[] robots = {null, new Robot(0), new Robot(1), new Robot(1)};
        Robot[] subArray = {robots[1], robots[2]};
        for (Robot occupier : robots) {
            for (int p0Crumb = 0; p0Crumb < 2; p0Crumb++) {
                for (int p1Crumb = 0; p1Crumb < 2; p1Crumb++) {
                    StreetCell c1 = newStreetCell(subArray, booleanVal(p0Crumb),
                            booleanVal(p1Crumb), occupier);
                    StreetCell c2 = newStreetCell(subArray, booleanVal(p0Crumb),
                            booleanVal(p1Crumb), occupier);
                    cells1.add(c1);
                    cells2.add(c2);
                }
            }
        }
    }

    //TODO Write test for getRobotTrails()
}
