package robots;

import backend.Move;
import backend.RobotAlgorithmIF;
import backend.SensorGroup;

public class PredictableTestRobot implements RobotAlgorithmIF {
    
    private enum Goal {MakeSquare, NextSquare, NextEvenRow, NextOddRow};

	private int movesMade = 0;
    private int rowTurn;
    private int moveTurn;
    private Goal goal = Goal.MakeSquare;

    public Move getMove(SensorGroup sensorData) {
		movesMade++;
        rowTurn++;
        moveTurn++;
        if (rowTurn == 25 * 14 - 1) {
            moveTurn = 1;
            goal = Goal.NextEvenRow;
        } if (rowTurn == 51 * 14 - 4) {
            moveTurn = 1;
            goal = Goal.NextOddRow;
        }
        
        if (goal == Goal.MakeSquare) {
            return makeSquareMove();
        } else if (goal == Goal.NextSquare) {
            return nextSquareMove();
        } else if (goal == Goal.NextEvenRow){
            return nextEvenRowMove();
        } else {
            return nextOddRowMove();
        }
        
    }

    private Move nextOddRowMove() {
        rowTurn = 0;
        moveTurn = 0;
        goal = Goal.MakeSquare;
        return Move.TURN_RIGHT;
    }

    private Move nextEvenRowMove() {
        if (moveTurn < 6) {
            return makeSquareMove();
        } else if (moveTurn < 8) {
            return Move.AHEAD_WITH_CRUMBS;
        } else if (moveTurn == 8) {
            return Move.TURN_RIGHT;
        } else {
            goal = Goal.MakeSquare;
            moveTurn = 1;
            return makeSquareMove();
        }
    }

    private Move nextSquareMove() {
        if (moveTurn % 3 != 0) {
            return Move.AHEAD_WITH_CRUMBS;
        } else {
            goal = Goal.MakeSquare;
            moveTurn = 1;
            return makeSquareMove();
        }
    }

    private Move makeSquareMove() {
        if (moveTurn % 3 == 0) {
            if (moveTurn % 12 == 0) {
                goal = Goal.NextSquare;
            }
            return Move.TURN_RIGHT;
        }
        return Move.AHEAD_WITH_CRUMBS;
    }

    public String getName() {
        return this.getClass().getSimpleName();
    }

    public String toString() {
        return getName();
    }

	public int getMovesMade() {
		return movesMade;
	}

}
