package testutils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;

public class TestUtils extends Assert {

	/**
	 * Given two lists list1 and list2, verifies that:
	 * list1.get(i).equals(list2.get(j)) if and only if (i == j) Similarly for
	 * hashCodes.
	 * 
	 * @param list1
	 * @param list2
	 */
	public static void assertEqualsAndHashCodes(List<?> list1, List<?> list2) {
	    assertEquals(list1, list2);
        assertHashCodes(list1, list2);
	}
    
    /**
     * Given two lists list1 and list2, verifies that:
     * list1.get(i).equals(list2.get(j)) if and only if (i == j).
     * Also verifies that if the two elements are equal, then
     * they have the same hash code.
     * 
     * @param list1
     * @param list2
     */
    public static void assertEquals(List<?> list1, List<?> list2) {
        for (int i = 0; i < list1.size(); i++) {
            for (int j = 0; j < list2.size(); j++) {
                Object c1 = list1.get(i);
                Object c2 = list2.get(j);
                String equalsMessage = String.format(
                        "The Objects should be equal were: <%s> and <%s>", c1,
                        c2);
                String hashMessage = String
                        .format(
                                "The Objects should have equal hash codes were: <%d> and <%d>",
                                c1.hashCode(), c2.hashCode());

                assertTrue(equalsMessage, (c1.equals(c2) == (i == j)));
                assertTrue(hashMessage, c1.hashCode() == c2.hashCode()
                        || i != j);
            }
        }
    }
    
    /**
     * Given two lists list1 and list2, verifies that:
     * list1.get(i).equals(list2.get(j)) if and only if their hashCodes
     * are equal.  May not necessarily be true, especially if the hashCode
     * of the objects being compared use Sets.
     * 
     * @param list1
     * @param list2
     */
    public static void assertHashCodes(List<?> list1, List<?> list2) {
        for (int i = 0; i < list1.size(); i++) {
            for (int j = 0; j < list2.size(); j++) {
                Object c1 = list1.get(i);
                Object c2 = list2.get(j);
                assertEquals(
                        "The hash codes of equal street cells should be equal" +
                        "; hashCodes: " + c1.hashCode() + ", " + c2.hashCode() + 
                        "; i = " + i + "; j = " + j +
                        "\n" + c1.toString() + ";\n" + c2.toString(),
                        c1.hashCode() == c2.hashCode(), i == j);
            }
        }
    }

	public static void assertArraysEqual(String message, Object[] expected,
			Object[] actual) {
		assertEquals(message, Arrays.asList(expected), Arrays.asList(actual));
	}

	public static void assertArraysEqual(String message, int[] expected,
			int[] actual) {
		String actualMessage = String.format("%s expected: <%s> but was: <%s>",
				message, Arrays.toString(expected), Arrays.toString(actual));
		assertTrue(actualMessage, Arrays.equals(expected, actual));
	}

	public static <T> void verifyRandomness(T[] results, T[] fullSet) {
		final double marginOfError = .35;

		Map<T, Integer> numOfEach = new HashMap<T, Integer>();
		for (T d : results) {
			int num = getNumber(numOfEach, d);
			numOfEach.put(d, num + 1);
		}
		T[] allDirections = fullSet;
		int expected = results.length / allDirections.length;
		for (T d : allDirections) {
			int num = getNumber(numOfEach, d);
			String message = String
					.format(
							"The directions should be approximately evenly distributed; was %d, but should have been around %d",
							num, expected);
			assertTrue(message, expected * (1 - marginOfError) < num
					&& num < (1 + marginOfError) * expected);
		}
	}

	public static <T> void verifyRandomness(List<T> results, List<T> fullSet) {
		verifyRandomness(results.toArray(), fullSet.toArray());
	}

	public static <T> int getNumber(Map<T, Integer> numOfEach, T element) {
		Integer num = numOfEach.get(element);
		return (num == null ? 0 : num);
	}

}
